/***
 * Cecilia ADL Compiler
 * Copyright (C) 2005 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.adl.bindings;

import java.util.Map;

/**
 * A builder interface to build internal bindings of composite components.
 */
public interface BindingBuilder {

  /** If the binding to build is a normal binding. */
  int NORMAL_BINDING      = 0x00;

  /** If the binding to build is an export binding. */
  int EXPORT_BINDING      = 0x01;

  /** If the binding to build is an import binding. */
  int IMPORT_BINDING      = 0x02;

  /**
   * The bit mask of {@link #NORMAL_BINDING}, {@link #EXPORT_BINDING} and
   * {@link #IMPORT_BINDING} types.
   */
  int NEI_BINDING_MASK    = 0x03;

  /** If the binding to build is a static binding. */
  int STATIC_BINDING      = 0x04;

  /** The bit mask of {@link #STATIC_BINDING} type. */
  int STATIC_BINDING_MASK = 0x04;

  /** If the binding to build is a 'bindafterstart'. */
  int AFTERSTART_BINDING  = 0x08;

  /** If the binding to build is a 'usedAtStart'. */
  int USED_AT_START       = 0x10;

  /**
   * The bit mask of {@link #AFTERSTART_BINDING} and {@link #USED_AT_START}
   * types.
   */
  int START_PROPERTY_MASK = 0x18;

  /**
   * Binds two components in a composite.
   * 
   * @param componentType the type of the composite in which the binding is
   *            made.
   * @param componentInstance the instance of the composite in which the binding
   *            is made.
   * @param bindingType the type of the binding ({@link #NORMAL_BINDING},
   *            {@link #EXPORT_BINDING} or {@link #IMPORT_BINDING}).
   * @param client the instance of the client component.
   * @param clientItf the name of the client interface.
   * @param server the instance of the server component.
   * @param serverItf the name of the server interface.
   * @param codePieces the code pieces that as been produced by builders this
   *            builder depends on.
   * @param context additional parameters.
   * @return the code produced by this builder.
   * @throws Exception if something wrong append.
   */
  Object bindComponent(Object componentType, Object componentInstance,
      int bindingType, Object client, String clientItf, Object server,
      String serverItf, Map<String, Object> codePieces,
      Map<Object, Object> context) throws Exception;

}
