/***
 * Cecilia ADL Compiler
 * Copyright (C) 2006-2007 STMicroelectronics
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: fractal@objectweb.org
 *
 * Author: Matthieu Leclercq
 */

package org.objectweb.fractal.cecilia.composite.c.types;

import java.util.Map;

import org.objectweb.fractal.adl.types.TypeInterface;
import org.objectweb.fractal.cecilia.adl.idl.util.CUtil;
import org.objectweb.fractal.cecilia.adl.interfaces.InterfaceBuilder;
import org.objectweb.fractal.task.codegen.util.CodeWriter;

/**
 * Builds the definition of the exported interfaces of a cloneable composite
 * component.
 */
public class CloneableExportedDefinitionBuilder implements InterfaceBuilder {

  // --------------------------------------------------------------------------
  // Implementation of the InterfaceBuilder interface
  // --------------------------------------------------------------------------

  public Object createInterfaces(final Object componentType,
      final Object componentInstance, final Object contentType,
      final TypeInterface[] interfaceTypes,
      final Map<String, Object> codePieces, final Map<Object, Object> context) {

    final CodeWriter cw = new CodeWriter(
        "Exported Interface Definition Builder");
    // include idl.h files
    for (final TypeInterface itf : interfaceTypes)
      cw.append(CUtil.buildInclude(itf.getSignature()));
    cw.endl();

    // Build the structure of provided interfaces
    cw.append("struct ").append(componentType.toString().replace('.', '_'))
        .append("_exporteds {").endl();
    cw.append("Rfractal_api_Component           component;").endl();
    cw.append("Rfractal_api_BindingController   binding_controller;").endl();
    cw.append("Rfractal_api_LifeCycleController lifecycle_controller;").endl();
    cw.append("Rfractal_api_ContentController   content_controller;").endl();
    cw.append("// The followong structure is their because of the current ")
        .endl();
    cw.append("// definition of a template composite component and should")
        .endl();
    cw.append("// be removed later").endl();
    cw.append("struct {").endl();
    cw.append("unsigned char              nbSubComponents;").endl();
    cw.append("unsigned char              nbBindings;").endl();
    cw.append("unsigned char              started;").endl();
    cw.append("struct SubComponentDesc_t* subComponents;").endl();
    cw.append("struct BindingDesc_t*      bindings;").endl();
    cw.append("} __padding__;").endl();
    cw.append("Rfractal_api_Factory factory;").endl();
    cw.append("};").endl();

    return cw.toString();
  }
}
